@@ -201,7 +201,7 @@ roots = function (f, vectorized=FALSE, interval, maxerror_f = 1e-07, split = "se
     I.roots = .lapply(I.list, function(I) {
         r = NULL
         try({
-            r = root(f, lower = I[1], upper = I[2], # = I[3], f_upper = I[4],
+            r = root(f, lower = I[1], upper = I[2], f_lower = I[3], f_upper = I[4],
                      maxerror_f = maxerror_f, tol = tol, ...)
         }, silent = F)
         r
@@ -233,7 +233,7 @@ min_dist <- function (x, X, norm=rep(1,ncol(X))){
 #' @param f Function (one or more dimensions) to find roots of
 #' @param vectorized boolean: is f already vectorized ? (default: FALSE) or if function: vectorized version of f.
 #' @param intervals bounds to inverse in, each column contains min and max of each dimension
-#' @param mesh function or "unif" or "seq" (default) to preform interval partition
+#' @param mesh.type function or "unif" or "seq" (default) to preform interval partition
 #' @param mesh.sizes number of parts for mesh (duplicate for each dimension if using "seq")
 #' @param vectorized is f already vectorized ? (default: no)
 #' @param maxerror_f the maximum error on f evaluation (iterates over uniroot to converge).
@@ -245,76 +245,86 @@ min_dist <- function (x, X, norm=rep(1,ncol(X))){
 #' @export
 #' @return matrix of x, so f(x)=0
 #' @examples
-#' mesh_roots(function(x) x-.51, intervals=rbind(0,1))
-#' mesh_roots(function(x) sum(x)-.51, intervals=cbind(rbind(0,1),rbind(0,1)))
-#' mesh_roots(sin,intervals=c(pi/2,5*pi/2))
-#' mesh_roots(f = function(x) sin(pi*x[1])*sin(pi*x[2]),
+#' roots_mesh(function(x) x-.51, intervals=rbind(0,1))
+#' roots_mesh(function(x) sum(x)-.51, intervals=cbind(rbind(0,1),rbind(0,1)))
+#' roots_mesh(sin,intervals=c(pi/2,5*pi/2))
+#' roots_mesh(f = function(x) sin(pi*x[1])*sin(pi*x[2]),
 #'            intervals = matrix(c(1/2,5/2,1/2,5/2),nrow=2))
 #'
-#' r = mesh_roots(f = function(x) (0.25+x[1])^2+(0.5+x[2])^2 - .25,
+#' r = roots_mesh(f = function(x) (0.25+x[1])^2+(0.5+x[2])^2 - .25,
 #' intervals=matrix(c(-1,1,-1,1),nrow=2), mesh.size=5)
 #' plot(r,xlim=c(-1,1),ylim=c(-1,1))
 #'
-#' r = mesh_roots(function(x) (0.5+x[1])^2+(-0.5+x[2])^2+(0.+x[3])^2 - .5,
+#' r = roots_mesh(function(x) (0.5+x[1])^2+(-0.5+x[2])^2+(0.+x[3])^2 - .5,
 #'                mesh.sizes = 11,
 #'                intervals=matrix(c(-1,1,-1,1,-1,1),nrow=2))
 #' scatterplot3d::scatterplot3d(r,xlim=c(-1,1),ylim=c(-1,1),zlim=c(-1,1))
 #'
-#' mesh_roots(function(x)exp(x)-1,intervals=c(-1,2))
-#' mesh_roots(function(x)exp(1000*x)-1,intervals=c(-1,2))
-mesh_roots = function (f, vectorized = FALSE, intervals, mesh = "seq", mesh.sizes = 11,
+#' roots_mesh(function(x)exp(x)-1,intervals=c(-1,2))
+#' roots_mesh(function(x)exp(1000*x)-1,intervals=c(-1,2))
+roots_mesh = function (f, vectorized = FALSE, intervals, mesh.type = "seq", mesh.sizes = 11,
                        maxerror_f = 1e-07, tol = .Machine$double.eps^0.25, ...) {
-  if (is.matrix(intervals)) {
-    lowers = apply(intervals, 2, min)
-    uppers = apply(intervals, 2, max)
+  # if (is.matrix(intervals)) {
+  #   lowers = apply(intervals, 2, min)
+  #   uppers = apply(intervals, 2, max)
+  #   d = ncol(intervals)
+  # } else if (is.null(intervals) && is.matrix(mesh)) {
+  #   lowers = apply(mesh, 2, min)
+  #   uppers = apply(mesh, 2, max)
+  #   d = ncol(mesh)
+  # } else
+  #   d = 1
+
+  if (is.matrix(intervals))
     d = ncol(intervals)
-  } else if (is.null(intervals) && is.matrix(mesh)) {
-    lowers = apply(mesh, 2, min)
-    uppers = apply(mesh, 2, max)
+  else if (is.null(intervals) && is.matrix(mesh.type))
     d = ncol(mesh)
-  } else
+  else
     d = 1
 
   if (d == 1)
-    return(roots(f, vectorized = vectorized, interval = intervals, split = mesh, split.size = mesh.sizes,
+    return(roots(f, vectorized = vectorized, interval = intervals, split = mesh.type, split.size = mesh.sizes,
                  maxerror_f = maxerror_f, tol = tol, ...))
 
-  if (length(mesh.sizes) != d)
-    mesh.size = rep(mesh.sizes, d)
-  if (is.matrix(mesh)) {
-    ridge.points = mesh
-  } else if (is.function(mesh)) {
-    ridge.points = mesh(prod(mesh.size), lowers, uppers)
-  } else if (mesh == "seq") {
-    ridge.points = matrix(seq(f = lowers[1], to = uppers[1],
-                              length.out = mesh.size[1]), ncol = 1)
-    for (i in 2:d) {
-      ridge.points = combn.design(ridge.points, matrix(seq(f = lowers[i], to = uppers[i], length.out = mesh.size[i]), ncol = 1))
-    }
-  } else if (mesh == "unif") {
-    ridge.points = matrix(runif(n = d * prod(mesh.size),
-                                min = 0, max = 1), ncol = d)
-    ridge.points = matrix(lowers, ncol = d, nrow = nrow(ridge.points),
-                          byrow = TRUE) +
-      ridge.points * (matrix(uppers, ncol = d, nrow = nrow(ridge.points), byrow = TRUE) -
-                        matrix(lowers, ncol = d, nrow = nrow(ridge.points), byrow = TRUE))
+  # if (length(mesh.sizes) != d)
+  #   mesh.size = rep(mesh.sizes, d)
+  # if (is.matrix(mesh)) {
+  #   ridge.points = mesh
+  # } else if (is.function(mesh)) {
+  #   ridge.points = mesh(prod(mesh.size), lowers, uppers)
+  # } else if (mesh == "seq") {
+  #   ridge.points = matrix(seq(f = lowers[1], to = uppers[1],
+  #                             length.out = mesh.size[1]), ncol = 1)
+  #   for (i in 2:d) {
+  #     ridge.points = combn.design(ridge.points, matrix(seq(f = lowers[i], to = uppers[i], length.out = mesh.size[i]), ncol = 1))
+  #   }
+  # } else if (mesh == "unif") {
+  #   ridge.points = matrix(runif(n = d * prod(mesh.size),
+  #                               min = 0, max = 1), ncol = d)
+  #   ridge.points = matrix(lowers, ncol = d, nrow = nrow(ridge.points),
+  #                         byrow = TRUE) +
+  #     ridge.points * (matrix(uppers, ncol = d, nrow = nrow(ridge.points), byrow = TRUE) -
+  #                       matrix(lowers, ncol = d, nrow = nrow(ridge.points), byrow = TRUE))
+  #
+  # } else if (mesh == "LHS") {
+  #   ridge.points = DiceDesign::lhsDesign(prod(mesh.size), dimension = d)$design
+  #   ridge.points = matrix(lowers, ncol = d, nrow = nrow(ridge.points), byrow = TRUE) +
+  #     ridge.points * (matrix(uppers, ncol = d, nrow = nrow(ridge.points), byrow = TRUE) -
+  #                       matrix(lowers, ncol = d, nrow = nrow(ridge.points), byrow = TRUE))
+  #
+  # } else stop("unsupported mesh : ", mesh)
+  #
+  # b = c(lowers[1], uppers[1])
+  # for (id in 1:(d - 1)) {
+  #   b = rbind(cbind(b, lowers[id + 1]), cbind(b, uppers[id + 1]))
+  # }
+  # for (i in 1:nrow(b)) if (min_dist(b[i, ], ridge.points) > .Machine$double.eps)
+  #   ridge.points = rbind(ridge.points, b[i, ])
+  #
+  # simplexes <- geometry::delaunayn(ridge.points, output.options = TRUE)
 
-  } else if (mesh == "LHS") {
-    ridge.points = DiceDesign::lhsDesign(prod(mesh.size), dimension = d)$design
-    ridge.points = matrix(lowers, ncol = d, nrow = nrow(ridge.points), byrow = TRUE) +
-      ridge.points * (matrix(uppers, ncol = d, nrow = nrow(ridge.points), byrow = TRUE) -
-                        matrix(lowers, ncol = d, nrow = nrow(ridge.points), byrow = TRUE))
-
-  } else stop("unsupported mesh : ", mesh)
-
-  b = c(lowers[1], uppers[1])
-  for (id in 1:(d - 1)) {
-    b = rbind(cbind(b, lowers[id + 1]), cbind(b, uppers[id + 1]))
-  }
-  for (i in 1:nrow(b)) if (min_dist(b[i, ], ridge.points) > .Machine$double.eps)
-    ridge.points = rbind(ridge.points, b[i, ])
-
-  simplexes <- geometry::delaunayn(ridge.points, output.options = TRUE)
+  simplexes = mesh(intervals, mesh.type, mesh.sizes)
+  ridge.points = simplexes$p
 
   if (isTRUE(vectorized) && is.function(f))
     f_vec = function(x, ...) f(x, ...)
@@ -327,9 +337,11 @@ mesh_roots = function (f, vectorized = FALSE, intervals, mesh = "seq", mesh.size
     stop("Cannot decide how to vectorize f")
 
   ridge.y = f_vec(ridge.points, ...)
+
   if (requireNamespace("arrangements"))
     tcombn2 = function(x) arrangements::combinations(x, 2)
   else tcombn2 = function(x) t(utils::combn(x, 2))
+
   ridges = NULL
   for (i in 1:nrow(simplexes$tri)) {
     more_ridges = tcombn2(simplexes$tri[i, ])
@@ -364,15 +376,15 @@ mesh_roots = function (f, vectorized = FALSE, intervals, mesh = "seq", mesh.size
   #     else return(NULL)
   # }, .combine = rbind)
 
-  r = do.call(rbind,mclapply(X = 1:nrow(ridges), FUN = function(ridge_i,...) {
+  r = do.call(rbind,parallel::mclapply(X = 1:nrow(ridges), FUN = function(ridge_i,...) {
     X = ridge.points[ridges[ridge_i,], , drop = FALSE]
     y = ridge.y[ridges[ridge_i,]]
     if (isFALSE((all(y > 0) || all(y < 0)))) {
       f_r = function(alpha, ...) {
-        as.numeric(f(alpha * X[1, ] + (1 - alpha) * X[2, ], ...))}
+        as.numeric(f(alpha * X[1, ] + (1 - alpha) * X[2, ], ...))} # assumes one only root (inside current mesh element)
       alpha_i = 0 # default value if root finding fails after:
       try({alpha_i <- root(f=f_r, lower = 0, upper = 1, maxerror_f = maxerror_f,
-                           tol = tol, f_lower = y[2], f_upper = y[1], ...)})
+                           tol = tol, f_lower = y[2], f_upper = y[1], max.rec=10, ...)})
       return(alpha_i * X[1, ,drop=FALSE] + (1 - alpha_i) * X[2, ,drop=FALSE])
     } else return(NULL)
   }))
@@ -389,3 +401,44 @@ mesh_roots = function (f, vectorized = FALSE, intervals, mesh = "seq", mesh.size
 
   return(r)
 }
+
+#' @title Mesh level set of function
+#' @param f function to be evaluated on the mesh
+#' @param vectorized logical or function. If TRUE, f is assumed to be vectorized.
+#' @param level level/threshold value
+#' @param intervals matrix of intervals
+#' @param mesh mesh object or type
+#' @param ... additional arguments passed to f
+#' @export
+mesh_level = function(f, vectorized=FALSE, level=0, intervals, mesh, ...) {
+  if (!is.mesh(mesh))
+    mesh = mesh(intervals, mesh.type=mesh)
+
+  if (is.array(f) && length(f)==nrow(mesh$p))
+    mesh$y = f
+  else {
+    if (isTRUE(vectorized))
+      f_vec=function(x,...) f(x,...)
+    else if (is.function(vectorized))
+      f_vec=function(x,...) vectorized(x,...)
+    else
+      f_vec=Vectorize.function(f,dim=ncol(mesh$p))
+    mesh$y = f_vec(mesh$p,...)
+  }
+  ## Benchmarking
+  # a = runif(5)-0.5
+  # microbenchmark::microbenchmark(
+  #   any(a>=0) && any(a<=0),
+  #   any(diff(sign(a))!=0),
+  #   times=100
+  # )
+  I = which(apply(mesh$tri,1,function(i) {yi=mesh$y[i]-level; any(yi>=0) && any(yi<=0)}))
+
+  colnames(mesh$p) <- colnames(intervals)
+
+  return(list(p=mesh$p,
+              y=mesh$y,
+              tri=mesh$tri[I,],
+              areas=mesh$areas[I],
+              neighbours=mesh$neighbours[I]))
+}
